import datetime
import importlib
import ipaddress
import os
import subprocess
from flask.json.provider import DefaultJSONProvider
import flask

import kitnet.lib.host_oper_mode_data
import kitnet.lib.netdd
import postgresql.types
from beaker.middleware import SessionMiddleware
from flask import Flask
from kitnet.lib.netdb import PostgreSQL_old as PostgreSQL

from .model import DBObject, MetaDBObject, SimpleKITUser

DIRTY = True

session_opts = {
    'session.type': 'file',
    'session.url': '127.0.0.1:5000',
    'session.data_dir': './cache',
}


class ModMetaData(object):
    def __init__(self, name, version, contact_email, printable_name, mod_path, gitlab_url=None, search_func=None,
                 search_opts_func=None):
        self.version = version
        self.contact_mail = contact_email
        self.gitlab_url = gitlab_url
        self.printable_name = printable_name
        self.mod_path = mod_path
        self.name = name
        self.search_func = search_func
        self.search_opts_func = search_opts_func if search_func is not None else None


class CustomJSONEncoder(DefaultJSONProvider):
    def default(self, obj):
        if isinstance(obj, DBObject) or isinstance(obj, SimpleKITUser):
            return {k: v for (k, v) in obj.__dict__.items() if not k.startswith('_')}
        elif isinstance(obj, ipaddress.IPv4Network) or isinstance(obj, ipaddress.IPv6Network):
            return {'address': str(obj), 'netmask': obj.netmask, 'broadcast_address': obj.broadcast_address}
        elif isinstance(obj, ipaddress.IPv4Address) or isinstance(obj, ipaddress.IPv6Address):
            return str(obj)
        elif isinstance(obj, datetime.datetime):
            return obj.isoformat()
        elif isinstance(obj, ModMetaData):
            return {k: v for (k, v) in obj.__dict__.items() if 'func' not in k}
        elif isinstance(obj, kitnet.lib.host_oper_mode_data.Loader):
            return {'is_devel': obj.OP_ENV_IS_DEVEL, 'is_prod': obj.OP_ENV_IS_PROD, 'is_test': obj.OP_ENV_IS_TEST,
                    'mode': obj.OP_ENV_MODE}
        elif isinstance(obj, postgresql.types.Array):
            return list(obj)
        print('Passing to default encoder:', obj.__class__)
        return JSONEncoder.default(self, obj)


class MyApp(Flask):
    pass


flask.Flask.json_provider_class = CustomJSONEncoder
# Define the WSGI application object
app = MyApp(__name__)

app.wsgi_app = SessionMiddleware(app.wsgi_app, session_opts)
# ldap_helper = kitnet.lib.idmldap.Helper(kitnet.lib.idmldap.CfgLdr())

db = PostgreSQL()
db_conn = db.connect()


def get_git_version(path):
    try:
        v = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'], cwd=os.path.dirname(path))
        v = v.strip()
        return v.decode('utf-8')
    except subprocess.CalledProcessError:
        return 'unknown'


VERSION = get_git_version(__file__)
MODS = list()


def load_mods():
    app.logger.info("Loading net-suite modules...")
    login_mod = importlib.import_module(app.config.get('LOGIN_MODULE'))
    app.register_blueprint(blueprint=getattr(login_mod, app.config.get('LOGIN_MODULE').split('.')[-1]), url_prefix='')
    modules = app.config.get('MODULES')
    for m in modules:
        mod = importlib.import_module(m['mod'])
        MODS.append(mod.METADATA)
        app.register_blueprint(blueprint=getattr(mod, m['mod'].split('.')[-1]),
                               url_prefix='{prefix}{url}'.format(prefix='', url=m['url']))


def finalize_init():
    db_conn.close()


from . import views
