from flask import render_template, request, redirect, abort, flash, jsonify
from net_suite import db
from net_suite.views import get_db_conn
from net_suite.model import *
from . import login_oic, app
from oic.oic.message import RegistrationResponse, AuthorizationResponse
from oic.oic import Client
from oic.utils.authn.client import CLIENT_AUTHN_METHOD
from oic import rndstr
from oic.utils.http_util import Redirect


def get_client():
    oic_client = Client(client_authn_method=CLIENT_AUTHN_METHOD)
    oic_provider_info = oic_client.provider_config(app.config.get('OIC_ENDPOINT'))
    info = {"client_id": app.config.get('OIC_CLIENT_ID'), "client_secret": app.config.get('OIC_CLIENT_SECRET')}
    client_reg = RegistrationResponse(**info)
    oic_client.store_registration_info(client_reg)
    return oic_client


def get_authorization_response(client):
    authorization_response = client.parse_response(
        AuthorizationResponse,
        info=request.args,
        sformat='dict')
    return authorization_response


@login_oic.route('/api/login', methods=['POST'])
def api_login():
    s = request.environ['beaker.session']
    if 'stay_logged_in' in request.json:
        s["wants_long_token"] = request.json['stay_logged_in']
    s.save()
    if 'login' not in s:
        oic_client = get_client()
        s["state"] = rndstr()
        s["nonce"] = rndstr()
        debug_t_host = request.environ.get('HTTP_ORIGIN', None)
        s.save()
        args = {
            "client_id": oic_client.client_id,
            "response_type": "code",
            "scope": ["openid", 'profile', 'email'],
            "nonce": s["nonce"],
            "redirect_uri": app.config.get('DEBUG_OIC_REDIR') + '/' + debug_t_host.split('//', 1)[
                1] if db.host_omdl.OP_ENV_IS_DEVEL and (request.host.split(':')[0] == 'localhost' or 'netvs-devel-oldrelease.scc.kit.edu' in request.host) else app.config.get(
                'OIC_REDIR'),
            "state": s["state"],
            'acr_values': 'phrh',
            "prompt": ["select_account"]
        }
        auth_req = oic_client.construct_AuthorizationRequest(request_args=args)
        login_url = auth_req.request(oic_client.authorization_endpoint)
        return jsonify({'status': 'usr_redir', 'url': login_url})
    login_token = None
    if not isinstance(s['login'], SimpleKITUser):
        login_token = s['login'].create_session_token(db, get_db_conn(), s.get("wants_long_token", False))
        s.save()
    return jsonify({'login': s['login'], 'token': login_token})


def user_from_userinfo(db, conn, userinfo):
    user = DBMgr.get_by_login_name(db, conn, userinfo.get('preferred_username', userinfo['email'].rsplit('@', 1)[0]))
    if user is not None:
        return user
    return SimpleKITUser(login_name=userinfo['preferred_username'],
                         first_name=userinfo.get('given_name', None),
                         last_name=userinfo.get('family_name', None),
                         email=userinfo['email'])


@login_oic.route('/api/oic_responder')
def oic_responder(t_host=None):
    s = request.environ['beaker.session']
    oic_client = get_client()
    aresp = get_authorization_response(oic_client)

    code = aresp["code"]
    assert aresp["state"] == request.args["state"]
    args = {
        "code": aresp['code'],
        "redirect_uri": app.config.get('OIC_REDIR'),
    }

    resp = oic_client.do_access_token_request(state=aresp['state'],
                                              request_args=args,
                                              authn_method='client_secret_basic')
    userinfo = oic_client.do_user_info_request(state=aresp["state"])
    s['login'] = user_from_userinfo(db, get_db_conn(), userinfo)
    s.save()
    return redirect('/oic_login')


@login_oic.route('/api/debug_oic_responder/<t_host>')
def debug_oic_responder(t_host=None):
    if not (db.host_omdl.OP_ENV_IS_DEVEL and t_host is not None):
        abort(404)

    oic_client = get_client()
    aresp = get_authorization_response(oic_client)
    code = aresp["code"]
    args = {
        "code": aresp['code'],
        "redirect_uri": app.config.get('DEBUG_OIC_REDIR') + '/' + t_host,
    }

    resp = oic_client.do_access_token_request(state=aresp['state'],
                                              request_args=args,
                                              authn_method='client_secret_basic')
    userinfo = oic_client.do_user_info_request(state=aresp["state"])
    new_target = 'http://' + t_host + '/api/debug_oic_receiver'
    return render_template('debug_forward.html', **{'redir': t_host, 'new_target': new_target, 'userinfo': userinfo})


@login_oic.route('/api/debug_oic_receiver', methods=['POST'])
def debug_oic_receiver():
    if not db.host_omdl.OP_ENV_IS_DEVEL:
        abort(404)
    s = request.environ['beaker.session']
    s['login'] = user_from_userinfo(db, get_db_conn(), request.form)
    s.save()
    return redirect('http://' + request.form['redir'] + '/oic_login')


@login_oic.route('/api/logout', methods=['POST'])
def api_logout():
    s = request.environ['beaker.session']
    if 'login' in s:
        if request.json.get('token_pk', None) is not None:
            APIToken(login_name=s['login'].login_name, pk=int(request.json['token_pk'])).delete(db, get_db_conn())
        s.delete()
    s.save()
    return jsonify({'logout': 'success'})
