import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import html
import jinja2
import pathlib

from flask import jsonify, request, current_app

from net_suite import app, db
from net_suite.views import api_netdb_required, get_current_mgr
from . import mail_requests


# Handle api request
@mail_requests.route('/send_patch_request/', methods=['POST'])
@api_netdb_required
def handle_request():
    try:
        return send_patch_request(request.json['actions'])
    except Exception as e:
        return jsonify({'error': str(e)})


def render_jinja_template(path, template, **kwargs):
    templateLoader = jinja2.FileSystemLoader(path)
    templateEnv = jinja2.Environment(loader=templateLoader, autoescape=True)  # autoescape everything
    templ = templateEnv.get_template(template)
    return templ.render(**kwargs)


def send_patch_request(actions):
    try:
        mgr = get_current_mgr()
        host_mode = db.host_omdl.OP_ENV_MODE.upper()

        path = pathlib.Path(__file__).parent.resolve()
        TEMPLATE = 'patch_request_template.j2'

        body = render_jinja_template(path, TEMPLATE,
                                     actions=actions,
                                     mgr=mgr,
                                     host_mode=host_mode
                                     )

        building_numbers = ",".join([action['building']['number'] for action in actions])

        subject = f'Patch Request: ({html.escape(building_numbers)}) for {html.escape(mgr.first_name)} {html.escape(mgr.last_name)}'
        if host_mode: subject = f'[{host_mode}] {subject}'

        send_email(to=current_app.config['PATCH_REQUEST_MAIL_RECEIVERS'],
                   sender=current_app.config['PATCH_REQUEST_MAIL_SENDER'],
                   replyTo=mgr.email,
                   subject=subject,
                   body=body
                   )
        return jsonify({'result': 'success'})
    except Exception as e:
        print(e)
        return jsonify({'result': 'error', 'message': repr(e)})


def send_email(to, sender, cc=None, bcc=None, replyTo=None, subject=None, body=None):
    if type(to) is not list:
        to = to.split()

    to_list = to + [cc] + [bcc]
    to_list = [x for x in to if x is not None]

    msg = MIMEMultipart('alternative')
    msg['From'] = sender
    msg['Subject'] = subject
    msg['To'] = ','.join(to)
    msg['Cc'] = cc
    msg['Bcc'] = bcc
    msg['X-Auto-Response-Suppress'] = 'All'
    msg['Auto-Submitted'] = 'yes'
    msg['Reply-To'] = replyTo
    msg.attach(MIMEText(body, 'html'))
    with smtplib.SMTP(app.config.get('MAIL_REQUESTS_SMARTHOST')) as server:
        server.sendmail(sender, to_list, msg.as_string())
